-----------------------------------------------------------------------------
_AUTO_RELOAD_DEBUG = true
-----------------------------------------------------------------------------
--[[ FILE INFO |
------------------------------------------------------------------------------

	Default Sample Settings Tool
	------------------------------------------
	Set defaults for new and imported samples.
	--
	DLT <dave.tichy@gmail.com>

	v0.1:
		- Sample interpolation mode and antialiasing
	
	TODO:
		- Enable support for all sample properties

------------------------------------------------------------------------------

	KNOWN BUGS:
	----------------
		- Here

---------------------------------------------------------------------------
Copyright © 2020 David Lopez Tichy, http://dlt.fm <dave.tichy@gmail.com>
---------------------------------------------------------------------------
The MIT License (MIT)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the “Software”), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
---------------------------------------------------------------------------

---------------------------------------------------------------------------]]

class 'Dlt_DefaultSampleSettings'
---------------------------------------------------------------------

function Dlt_DefaultSampleSettings:__init ()

	self.gui = nil

	-- Settings -----------------------------
	self.options = renoise.Document.create('DefaultSampleSettings') {
		-- These are Renoise defaults
		loop_mode = 1, -- off
		trigger_oneshot = false, 
		interpolation_mode = 3, -- cubic 
		interpolation_antialiasing = false, 
	}
	renoise.tool().preferences = self.options

	-- KB Shortcuts and menu entries
	self:init_kb_shortcuts()
	self:init_menu_entries()

	-- Automatic default settings on new/dragged samples
	renoise.tool().app_new_document_observable:add_notifier(function() self:init_triggers() end)
end


---------------------------------------------------------------------

function Dlt_DefaultSampleSettings:init_triggers ()

	local apply_defaults_sample = function ( opts ) 
		self:apply_defaults_to_sample ( opts.index )
	end

	renoise.song().selected_instrument_observable:add_notifier(function(opts) 
		if renoise.song().selected_instrument.samples_observable:has_notifier(apply_defaults_sample) then
			renoise.song().selected_instrument.samples_observable:remove_notifier(apply_defaults_sample)
		end
		renoise.song().selected_instrument.samples_observable:add_notifier(apply_defaults_sample)
	end)

end

---------------------------------------------------------------------

function Dlt_DefaultSampleSettings:init_kb_shortcuts ()
	
	renoise.tool():add_keybinding {
		name = "Sample Editor:Default Sample Settings:Apply to Current Sample",
		invoke = function() self:apply_defaults_to_current_sample() end
	}
	renoise.tool():add_keybinding {
		name = "Sample Editor:Default Sample Settings:Apply to Selected Instrument",
		invoke = function() self:apply_defaults_to_selected_instrument() end
	}


end

---------------------------------------------------------------------

function Dlt_DefaultSampleSettings:init_menu_entries ()
	
	renoise.tool():add_menu_entry {
		name = "Sample Navigator:Default Sample Settings:Apply to Current Sample",
		invoke = function() self:apply_defaults_to_current_sample() end
	}
	renoise.tool():add_menu_entry {
		name = "Sample Navigator:Default Sample Settings:Apply to All Samples in Instrument",
		invoke = function() self:apply_defaults_to_selected_instrument() end
	}
	renoise.tool():add_menu_entry {
		name = "--Sample Navigator:Default Sample Settings:Change Defaults...",
		invoke = function() self:open_gui() end
	}

	renoise.tool():add_menu_entry {
		name = "Instrument Box:Default Sample Settings:Apply to Selected Instrument",
		invoke = function() self:apply_defaults_to_selected_instrument() end
	}
	renoise.tool():add_menu_entry {
		name = "--Instrument Box:Default Sample Settings:Change Defaults...",
		invoke = function() self:open_gui() end
	}

end

---------------------------------------------------------------------

function Dlt_DefaultSampleSettings:apply_defaults_to_current_sample ()

	local id = renoise.song().selected_sample_index
	if id ~= 0 then
		self:apply_defaults_to_sample(id)
		renoise.app():show_status("Settings applied.")
	else
		renoise.app():show_status("No sample selected, or no samples to be changed.")
	end

end

---------------------------------------------------------------------

function Dlt_DefaultSampleSettings:apply_defaults_to_sample ( id )
	local sample = renoise.song().selected_instrument.samples[id]

	local valid_aa = {
		renoise.Sample.INTERPOLATE_NONE,
		renoise.Sample.INTERPOLATE_LINEAR,
		renoise.Sample.INTERPOLATE_CUBIC,
		renoise.Sample.INTERPOLATE_SINC
	}
	local aa = self.options.interpolation_mode.value
	sample.interpolation_mode = valid_aa[aa]
	sample.oversample_enabled = self.options.interpolation_antialiasing.value

	print('fired')
end

---------------------------------------------------------------------

function Dlt_DefaultSampleSettings:apply_defaults_to_selected_instrument ()
	local samples = renoise.song().selected_instrument.samples
	for i=1, #samples do
		self:apply_defaults_to_sample(i)
	end
	renoise.app():show_status("Settings applied.")
end

---------------------------------------------------------------------

function Dlt_DefaultSampleSettings:open_gui ()

	if self.gui and self.gui.visible then
		self.gui:show()
		return
	end

	self.vb = renoise.ViewBuilder()
	local vb = self.vb

	local option_dropdown = function ( option_id, items_table )
		return vb:popup {
			bind = self.options[option_id],
			items = items_table,
		}
	end

	local option_checkbox = function ( option_id )
		return vb:checkbox {
			bind = self.options[option_id],
			value = self.options[option_id].value,
		}
	end

	local content = vb:column {
		margin = renoise.ViewBuilder.DEFAULT_DIALOG_MARGIN,
		uniform=true,

		vb:text { style='strong',text='Interpolation' },
		vb:row {
			uniform=true,
			vb:column {
				vb:text { text='Antialiasing' },
				vb:text { text='Interpolation Mode' },
			},
			vb:column {
				option_checkbox('interpolation_antialiasing'),
				option_dropdown('interpolation_mode', {'None','Linear','Cubic','Sinc'}),
			},
		},
		vb:space { height=renoise.ViewBuilder.DEFAULT_DIALOG_SPACING },
		vb:row {
			vb:text { style='strong',text='Apply Now to' },
			vb:button {
				text='Current Sample',
				notifier=function() self:apply_defaults_to_current_sample() end
			},
			vb:button {
				text='All Samples in Instrument',
				notifier=function() self:apply_defaults_to_selected_instrument() end
			}
		}
	}

	self.gui = renoise.app():show_custom_dialog('Default Sample Settings', content)
end



-- INITIALIZE
local DSS = Dlt_DefaultSampleSettings()


-- TESTING
-- DSS:open_gui()

